function ApplovinAds () {
	var AppLovinMAX = null;

	const ADS_STATE = {
		INITIAL: 0,
		BEGIN_LOAD: 1,
		LOADED: 2,
		BEGIN_SHOW: 3,
	};

	var _initialized = false;

	var _banner_is_visible_logic = false;
	var _banner_is_created = false;

	var _reward_retry_attempt = 0;
	var _reward_state = ADS_STATE.INITIAL;

	var _interstitial_retry_attempt = 0;
	var _interstitial_state = ADS_STATE.INITIAL;

	function initialize() {
		// Повторная инициализация создаёт проблемы: повторные обработчики событий. 
		// Нужно учесть рефреш игры внутри фрейма.
		if (AppLovinMAX) {
			// Это аварийная ситуация, такого быть не должно, колбек не вызываем, лучше какой-то assert сделать
			return; 
		}

		AppLovinMAX = cordova.require('cordova-plugin-applovin-max.AppLovinMAX');
		// AppLovinMAX.setVerboseLogging(true); // Это нужно только для дебажной версии
		extraParams = {};
		Object.assign(extraParams, config.amazon_extra_config);
		AppLovinMAX.initialize(config.applovin.SDK_KEY, extraParams, function() {
			console.log('ApplovinAds initialized');
			
			_setup_rewarded_handlers();
			_setup_interstitial_handlers();
			_setup_banner_handlers();

			_initialized = true;
			// По мере готовности без запроса
			application.executeChild('Applovin_ready');
		});
	}

	function show_mediation_debugger () {
		AppLovinMAX.showMediationDebugger();
	}

	// REWARDED
	function rewarded_load () {
		if (_reward_state == ADS_STATE.INITIAL) {
			_reward_state = ADS_STATE.BEGIN_LOAD;
			AppLovinMAX.loadRewardedAd(config.applovin.REWARDED_AD_UNIT_ID);
		}
	}
	
	function rewarded_show () {
		if (_reward_state == ADS_STATE.LOADED) {
			_reward_state = ADS_STATE.BEGIN_SHOW;
			AppLovinMAX.showRewardedAd(config.applovin.REWARDED_AD_UNIT_ID);
		}
	}

	function _setup_rewarded_handlers () {
		window.addEventListener('OnRewardedAdLoadedEvent', function(adInfo) {
			console.log('OnRewardedAdLoadedEvent');
			_reward_retry_attempt = 0;
			_reward_state = ADS_STATE.LOADED;
			application.executeChild('Applovin_rewarded_loaded');
		});

		window.addEventListener('OnRewardedAdLoadFailedEvent', function(adInfo) {
			console.log('OnRewardedAdLoadFailedEvent');
			_reward_state = ADS_STATE.INITIAL;
			_reward_retry_attempt++;
			retryDelay = 1000 * Math.pow(2, Math.min(6, _reward_retry_attempt));
			setTimeout(rewarded_load, retryDelay);
		});

		window.addEventListener('OnRewardedAdFailedToDisplayEvent', function(adInfo) {
			console.log('OnRewardedAdFailedToDisplayEvent');
			if (_reward_state == ADS_STATE.BEGIN_SHOW) {
				_reward_state = ADS_STATE.INITIAL;
				setTimeout(rewarded_load, 2000);
				application.executeChild('Applovin_rewarded_close');
			}
		});
		
		window.addEventListener('OnRewardedAdHiddenEvent', function(adInfo) {
			console.log('OnRewardedAdHiddenEvent');
			if (_reward_state == ADS_STATE.BEGIN_SHOW) {
				_reward_state = ADS_STATE.INITIAL;
				setTimeout(rewarded_load, 2000);
				if (adInfo && typeof(adInfo.revenue) == 'number' && adInfo.revenue > 0.0001) {
					// VK Ads показывает в ревардах объявление сразу с крестиком и срабатывает этот вариант, может из-за тестовой рекламы
					application.executeChild('Applovin_rewarded_close', {reward: true});
				} else {
					application.executeChild('Applovin_rewarded_close');
				}
			}
		});

		window.addEventListener('OnRewardedAdReceivedRewardEvent', function(adInfo) {
			console.log('OnRewardedAdReceivedRewardEvent');
			if (_reward_state == ADS_STATE.BEGIN_SHOW) {
				_reward_state = ADS_STATE.INITIAL;
				application.executeChild('Applovin_rewarded_close', {reward: true});
				setTimeout(rewarded_load, 2000);
			}
		});
	}

	// BANNERS
	function _setup_banner_handlers () {
		window.addEventListener('OnBannerAdLoadedEvent', function (adInfo) {
			console.log("OnBannerAdLoadedEvent");
			if (_banner_is_visible_logic) {
				// Показываем баннер если должны, а если его закрыли и прилетел этот ивент то не показываем.
				setTimeout(banner_show, 1000); // Разносим по времени загрузку и показ, чтобы уменьшить подтормаживание
			}
		});
		
		window.addEventListener('OnBannerAdLoadFailedEvent', function (adInfo) {
			console.log("OnBannerAdLoadFailedEvent");
		});

		banner_height = AppLovinMAX.isTablet() ? '93px' : '53px';
		bottom_banner = document.getElementById('bottom_banner');
		if (bottom_banner) {
			bottom_banner.style.height = banner_height;
		}
	}

	function banner_show () {
		_banner_is_visible_logic = true;

		if (_banner_is_created) {
			AppLovinMAX.showBanner(config.applovin.BANNER_AD_UNIT_ID);
		
			banner = document.getElementById('bottom_banner')
			if (banner && banner.style.display != 'block') {
				banner.style.display = 'block'
			}
		} else {
			_banner_is_created = true;

			// Banners are automatically sized to 320x50 on phones and 728x90 on tablets
			// You may use the utility method `AppLovinMAX.isTablet()` to help with view sizing adjustments
			AppLovinMAX.createBanner(config.applovin.BANNER_AD_UNIT_ID, AppLovinMAX.AdViewPosition.BOTTOM_CENTER);
		
			// Set background or background color for banners to be fully functional
			// In this case we are setting it to black - PLEASE USE HEX STRINGS ONLY
			AppLovinMAX.setBannerBackgroundColor(config.applovin.BANNER_AD_UNIT_ID, '#000000');
		} 
	}

	function banner_hide () {
		_banner_is_visible_logic = false;
		
		if (_banner_is_created) {
			AppLovinMAX.hideBanner(config.applovin.BANNER_AD_UNIT_ID);
			
			banner = document.getElementById('bottom_banner')
			if (banner && banner.style.display == 'block') {
				banner.style.display = ''
			}
		}
	}

	// INTERSTITIAL
	function _setup_interstitial_handlers () {
		window.addEventListener('OnInterstitialLoadedEvent', function(adInfo) {
			console.log('OnInterstitialLoadedEvent');
			_interstitial_retry_attempt = 0;
			_interstitial_state = ADS_STATE.LOADED;
			application.executeChild('Applovin_interstitial_loaded');
		});

		window.addEventListener('OnInterstitialLoadFailedEvent', function(adInfo) {
			console.log('OnInterstitialLoadFailedEvent');
			_interstitial_state = ADS_STATE.INITIAL;
			_interstitial_retry_attempt++;
			retryDelay = 1000 * Math.pow(2, Math.min(6, _interstitial_retry_attempt));
			setTimeout(interstitial_load, retryDelay);
		});
		
		window.addEventListener('OnInterstitialAdFailedToDisplayEvent', function(adInfo) {
			console.log('OnInterstitialAdFailedToDisplayEvent');
			_interstitial_state = ADS_STATE.INITIAL;
			setTimeout(interstitial_load, 2000);
			application.executeChild('Applovin_interstitial_close');
		});
		
		window.addEventListener('OnInterstitialHiddenEvent', function(adInfo) {
			console.log('OnInterstitialHiddenEvent');
			_interstitial_state = ADS_STATE.INITIAL;
			setTimeout(interstitial_load, 2000);
			application.executeChild('Applovin_interstitial_close');
		});
	}

	function interstitial_load () {
		if (_interstitial_state == ADS_STATE.INITIAL) {
			_interstitial_state = ADS_STATE.BEGIN_LOAD;
			AppLovinMAX.loadInterstitial(config.applovin.INTER_AD_UNIT_ID);
		}
	}

	function interstitial_show () {
		if (_interstitial_state == ADS_STATE.LOADED) {
			_interstitial_state = ADS_STATE.BEGIN_SHOW;
			AppLovinMAX.showInterstitial(config.applovin.INTER_AD_UNIT_ID);
		}
	}

	// Регистириуем обработчики событий в application
	if (application.messages) {
		application.messages.push({ type: 'applovin_rewarded_load', callback: rewarded_load});
		application.messages.push({ type: 'applovin_rewarded_show', callback: rewarded_show});
		application.messages.push({ type: 'applovin_banner_show', callback: banner_show});
		application.messages.push({ type: 'applovin_banner_hide', callback: banner_hide});
		application.messages.push({ type: 'applovin_interstitial_load', callback: interstitial_load});
		application.messages.push({ type: 'applovin_interstitial_show', callback: interstitial_show});
		application.messages.push({ type: 'applovin_show_mediation_debugger', callback: show_mediation_debugger});
		
		application.messages.push({ type: 'applovin_does_exists', callback: function() {
			application.executeChild('Applovin_exists'); 
		} });

		application.messages.push({ type: 'applovin_is_ready', callback: function() {
			if (_initialized) {
				// Если готов прямо сейчас
				application.executeChild('Applovin_ready');
			} 
		} });
	}

	return {
		'initialize': initialize,
		'show_mediation_debugger': show_mediation_debugger,
		'rewarded_load': rewarded_load,
		'rewarded_show': rewarded_show,
		'banner_show': banner_show,
		'banner_hide': banner_hide,
		'interstitial_load': interstitial_load,
		'interstitial_show': interstitial_show,
	}
}
